@@ -5,6 +5,8 @@ common parameter transformations, and pre-defined sets of parameter
 specifications.
 """
 
-from .sedmodel import ProspectorParams, SedModel, SpecModel
+from .sedmodel import ProspectorParams, SedModel, SpecModel, PolySpecModel, SplineSpecModel
 
-__all__ = ["SpecModel", "SedModel", "ProspectorParams"]
+__all__ = ["ProspectorParams",
+           "SpecModel", "PolySpecModel", "SplineSpecModel",
+           "SedModel"]
