@@ -7,18 +7,20 @@ observed spectra and photometry from them, given a Source object.
 
 import numpy as np
 import os
+
 from numpy.polynomial.chebyshev import chebval, chebvander
-from .parameters import ProspectorParams
+from scipy.interpolate import splrep, BSpline
 from scipy.stats import multivariate_normal as mvn
 
 from sedpy.observate import getSED
 
+from .parameters import ProspectorParams
 from ..sources.constants import to_cgs_at_10pc as to_cgs
 from ..sources.constants import cosmo, lightspeed, ckms, jansky_cgs
 from ..utils.smoothing import smoothspec
 
 
-__all__ = ["SpecModel", "PolySpecModel",
+__all__ = ["SpecModel", "PolySpecModel", "SplineSpecModel",
            "SedModel", "PolySedModel", "PolyFitModel"]
 
 
@@ -33,6 +35,40 @@ class SpecModel(ProspectorParams):
     and penalties for marginalization over emission line amplitudes.
     """
 
+    def __init__(self, *args, **kwargs):
+
+        super().__init__(*args, **kwargs)
+
+        # get the emission line info
+        try:
+            SPS_HOME = os.getenv('SPS_HOME')
+            info = np.genfromtxt(os.path.join(SPS_HOME, 'data', 'emlines_info.dat'),
+                                 dtype=[('wave', 'f8'), ('name', '<U20')],
+                                 delimiter=',')
+            self.emline_info = info
+            self._use_eline = np.ones(len(info), dtype=bool)
+        except(OSError, KeyError, ValueError) as e:
+            print("Could not read and cache emission line info from $SPS_HOME/data/emlines_info.dat")
+            self.emline_info = e
+
+    def _available_parameters(self):
+        new_pars = [("sigma_smooth", ""),
+                    ("marginalize_elines", ""),
+                    ("elines_to_fit", ""),
+                    ("elines_to_fix", ""),
+                    ("elines_to_ignore", ""),
+                    ("eline_delta_zred", ""),
+                    ("eline_sigma", ""),
+                    ("use_eline_priors", ""),
+                    ("eline_prior_width", "")]
+        relevant_pars = [("mass", ""),
+                         ("lumdist", ""),
+                         ("zred", ""),
+                         ("nebemlineinspec", ""),
+                         ("add_neb_emission")]
+
+        return new_pars
+
     def predict(self, theta, obs=None, sps=None, sigma_spec=None, **extras):
         """Given a ``theta`` vector, generate a spectrum, photometry, and any
         extras (e.g. stellar mass), including any calibration effects.
@@ -79,29 +115,38 @@ class SpecModel(ProspectorParams):
 
         # generate spectrum and photometry for likelihood
         # predict_spec should be called before predict_phot
+        # because in principle it can modify the emission line parameters
+        # and also needs some things done in 'cache_eline_parameters`
+        # especially _ewave_obs and _use_elines
         spec = self.predict_spec(obs, sigma_spec)
-        phot = self.predict_phot(obs['filters'])
+        phot = self.predict_phot(obs.get('filters', None))
 
         return spec, phot, self._mfrac
 
     def predict_spec(self, obs, sigma_spec, **extras):
         """Generate a prediction for the observed spectrum.  This method assumes
         that the parameters have been set and that the following attributes are
-        present and correct:
+        present and correct
+
           + ``_wave`` - The SPS restframe wavelength array
           + ``_zred`` - Redshift
           + ``_norm_spec`` - Observed frame spectral fluxes, in units of maggies
           + ``_eline_wave`` and ``_eline_lum`` - emission line parameters from the SPS model
 
         It generates the following attributes
+
           + ``_outwave`` - Wavelength grid (observed frame)
           + ``_speccal`` - Calibration vector
-          + ``_elinespec`` - emission line spectrum
-          + ``_sed`` - Intrinsic spectrum (before cilbration vector applied)
+          + ``_sed`` - Intrinsic spectrum (before cilbration vector applied but including emission lines)
 
-        And if emission line marginalization is being performed, numerous
-        quantities related to the emission lines are also cached
-        (see ``get_el()`` for details.)
+        And the following attributes are generated if nebular lines are added
+
+          + ``_fix_eline_spec`` - emission line spectrum for fixed lines, intrinsic units
+          + ``_fix_eline_spec`` - emission line spectrum for fitted lines, with
+            spectroscopic calibration factor included.
+
+        Numerous quantities related to the emission lines are also cached (see
+        ``cache_eline_parameters()`` and ``get_el()`` for details.)
 
         :param obs:
             An observation dictionary, containing the output wavelength array,
@@ -125,30 +170,32 @@ class SpecModel(ProspectorParams):
         if self._outwave is None:
             self._outwave = obs_wave
 
-        # cache eline parameters
+        # --- cache eline parameters ---
         self.cache_eline_parameters(obs)
 
-        # smooth and put on output wavelength grid
+        # --- smooth and put on output wavelength grid ---
         smooth_spec = self.smoothspec(obs_wave, self._norm_spec)
 
-        # calibration
+        # --- add fixed lines if necessary ---
+        emask = self._fix_eline_pixelmask
+        if emask.any():
+            inds = self._fix_eline & self._valid_eline
+            espec = self.predict_eline_spec(line_indices=inds,
+                                            wave=self._outwave[emask])
+            self._fix_eline_spec = espec
+            smooth_spec[emask] += self._fix_eline_spec.sum(axis=1)
+
+        # --- calibration ---
         self._speccal = self.spec_calibration(obs=obs, spec=smooth_spec, **extras)
         calibrated_spec = smooth_spec * self._speccal
 
-        # generate (after fitting) the emission line spectrum
-        emask = self._eline_wavelength_mask
-        # If we're marginalizing over emission lines, and at least one pixel
-        # has an emission line in it
-        if self.params.get('marginalize_elines', False) & (emask.any()):
-            self._elinespec = self.get_el(obs, calibrated_spec, sigma_spec)
-            calibrated_spec[emask] += self._elinespec.sum(axis=1)
-        # Otherwise, if FSPS is not adding emission lines to the spectrum, we
-        # add emission lines to valid pixels here.
-        elif (self.params.get("nebemlineinspec", True) == False) & (emask.any()):
-            self._elinespec = self.get_eline_spec(wave=self._outwave[emask])
-            if emask.any():
-                calibrated_spec[emask] += self._elinespec.sum(axis=1)
+        # --- fit and add lines if necessary ---
+        emask = self._fit_eline_pixelmask
+        if emask.any():
+            self._fit_eline_spec = self.get_el(obs, calibrated_spec, sigma_spec)
+            calibrated_spec[emask] += self._fit_eline_spec.sum(axis=1)
 
+        # --- cache intrinsic spectrum ---
         self._sed = calibrated_spec / self._speccal
 
         return calibrated_spec
@@ -160,12 +207,13 @@ class SpecModel(ProspectorParams):
           + ``_wave`` - The SPS restframe wavelength array
           + ``_zred`` - Redshift
           + ``_norm_spec`` - Observed frame spectral fluxes, in units of maggies.
-          + ``_eline_wave`` and ``_eline_lum`` - emission line parameters from the SPS model
-
+          + ``_ewave_obs`` and ``_eline_lum`` - emission line parameters from
+            the SPS model
 
         :param filters:
-            List of :py:class:`sedpy.observate.Filter` objects.
-            If there is no photometry, ``None`` should be supplied
+            Instance of :py:class:`sedpy.observate.FilterSet` or list of
+            :py:class:`sedpy.observate.Filter` objects. If there is no
+            photometry, ``None`` should be supplied.
 
         :returns phot:
             Observed frame photometry of the model SED through the given filters.
@@ -178,46 +226,16 @@ class SpecModel(ProspectorParams):
         # generate photometry w/o emission lines
         obs_wave = self.observed_wave(self._wave, do_wavecal=False)
         flambda = self._norm_spec * lightspeed / obs_wave**2 * (3631*jansky_cgs)
-        mags = getSED(obs_wave, flambda, filters)
-        phot = np.atleast_1d(10**(-0.4 * mags))
+        phot = 10**(-0.4 * np.atleast_1d(getSED(obs_wave, flambda, filters)))
+        # TODO: below is faster for sedpy > 0.2.0
+        #phot = np.atleast_1d(getSED(obs_wave, flambda, filters, linear_flux=True))
 
         # generate emission-line photometry
-        if self.params.get('nebemlineinspec', False) == False:
+        if (self._want_lines & self._need_lines):
             phot += self.nebline_photometry(filters)
 
         return phot
 
-    def nebline_photometry(self, filters):
-        """Compute the emission line contribution to photometry.  This requires
-        several cached attributes:
-          + ``_ewave_obs``
-          + ``_eline_lum``
-
-        :param filters:
-            List of :py:class:`sedpy.observate.Filter` objects
-
-        :returns nebflux:
-            The flux of the emission line through the filters, in units of
-            maggies. ndarray of shape ``(len(filters),)``
-        """
-        elams = self._ewave_obs
-        # We have to remove the extra (1+z) since this is flux, not a flux density
-        # Also we convert to cgs
-        elums = self._eline_lum * self.flux_norm() / (1 + self._zred) * (3631*jansky_cgs)
-
-        # loop over filters
-        flux = np.zeros(len(filters))
-        for i, filt in enumerate(filters):
-            # calculate transmission at line wavelengths
-            trans = np.interp(elams, filt.wavelength, filt.transmission,
-                              left=0., right=0.)
-            # include all lines where transmission is non-zero
-            idx = (trans > 0)
-            if True in idx:
-                flux[i] = (trans[idx]*elams[idx]*elums[idx]).sum() / filt.ab_zero_counts
-
-        return flux
-
     def flux_norm(self):
         """Compute the scaling required to go from Lsun/Hz/Msun to maggies.
         Note this includes the (1+z) factor required for flux densities.
@@ -238,19 +256,81 @@ class SpecModel(ProspectorParams):
 
         return mass * unit_conversion / dfactor
 
+    @property
+    def _need_lines(self):
+        return (not (bool(self.params.get("nebemlineinspec", True))))
+
+    @property
+    def _want_lines(self):
+        return bool(self.params.get('add_neb_emission', False))
+
+    def nebline_photometry(self, filters, elams=None, elums=None):
+        """Compute the emission line contribution to photometry.  This requires
+        several cached attributes:
+          + ``_ewave_obs``
+          + ``_eline_lum``
+
+        :param filters:
+            Instance of :py:class:`sedpy.observate.FilterSet` or list of
+            :py:class:`sedpy.observate.Filter` objects
+
+        :param elams: (optional)
+            The emission line wavelength in angstroms.  If not supplied uses the
+            cached ``_ewave_obs`` attribute.
+
+        :param elums: (optional)
+            The emission line flux in erg/s/cm^2.  If not supplied uses  the
+            cached ``_eline_lum`` attribute and applies appropriate distance
+            dimming and unit conversion.
+
+        :returns nebflux:
+            The flux of the emission line through the filters, in units of
+            maggies. ndarray of shape ``(len(filters),)``
+        """
+        if (elams is None) or (elums is None):
+            elams = self._ewave_obs[self._use_eline]
+            # We have to remove the extra (1+z) since this is flux, not a flux density
+            # Also we convert to cgs
+            self.line_norm = self.flux_norm() / (1 + self._zred) * (3631*jansky_cgs)
+            elums = self._eline_lum[self._use_eline] * self.line_norm
+
+        # loop over filters
+        flux = np.zeros(len(filters))
+        try:
+            # TODO: Since in this case filters are on a grid, there should be a
+            # faster way to look up the transmission than the later loop
+            flist = filters.filters
+        except(AttributeError):
+            flist = filters
+        for i, filt in enumerate(flist):
+            # calculate transmission at line wavelengths
+            trans = np.interp(elams, filt.wavelength, filt.transmission,
+                              left=0., right=0.)
+            # include all lines where transmission is non-zero
+            idx = (trans > 0)
+            if True in idx:
+                flux[i] = (trans[idx]*elams[idx]*elums[idx]).sum() / filt.ab_zero_counts
+
+        return flux
+
     def cache_eline_parameters(self, obs, nsigma=5):
         """ This computes and caches a number of quantities that are relevant
         for predicting the emission lines, and computing the MAP values thereof,
         including
           + ``_ewave_obs`` - Observed frame wavelengths (AA) of all emission lines.
           + ``_eline_sigma_kms`` - Dispersion (in km/s) of all the emission lines
-          + ``_elines_to_fit`` - If fitting and marginalizing over emission lines,
-            this stores indices of the lines to actually fit, as a boolean
-            array. Only lines that are within ``nsigma`` of an observed
-            wavelength points are included.
-          + ``_eline_wavelength_mask`` - A mask of the `_outwave` vector that
+          + ``_fit_eline`` - If fitting and marginalizing over emission lines,
+            this stores a boolean mask of the lines to actually fit. Only lines
+            that are within ``nsigma`` of an observed wavelength points are
+            included.
+          + ``_fix_eline`` - this stores a boolean mask of the lines that are
+            to be added with the cloudy amplitudes Only lines that are within
+            ``nsigma`` of an observed wavelength point are included.
+          + ``_fit_eline_pixelmask`` - A mask of the `_outwave` vector that
             indicates which pixels to use in the emission line fitting.
             Only pixels within ``nsigma`` of an emission line are used.
+          + ``_fix_eline_pixelmask`` - A mask of the `_outwave` vector that
+            indicates which pixels to use in the fixed emission line prediction.
 
         Can be subclassed to add more sophistication
         redshift - first looks for ``eline_delta_zred``, and defaults to ``zred``
@@ -260,45 +340,73 @@ class SpecModel(ProspectorParams):
             Number of sigma from a line center to use for defining which lines
             to fit and useful spectral elements for the fitting.  float.
         """
+
         # observed wavelengths
         eline_z = self.params.get("eline_delta_zred", 0.0)
         self._ewave_obs = (1 + eline_z + self._zred) * self._eline_wave
 
+        # masks for lines to be treated in various ways.
+        # always run this becuase it's need for spec *and* phot if adding lines
+        # by hand
+        self.parse_elines()
+
+        # exit gracefully if not adding lines.  We also exit if only fitting
+        # photometry, for performance reasons
+        hasspec = obs.get('spectrum', None) is not None
+        if not (self._want_lines & self._need_lines & hasspec):
+            self._fit_eline = None
+            self._fit_eline_pixelmask = np.array([], dtype=bool)
+            self._fix_eline_pixelmask = np.array([], dtype=bool)
+            self._fix_eline = None
+            return
+
         # observed linewidths
         nline = self._ewave_obs.shape[0]
         self._eline_sigma_kms = np.atleast_1d(self.params.get('eline_sigma', 100.0))
+        # what is this wierd construction for?
         self._eline_sigma_kms = (self._eline_sigma_kms[None] * np.ones(nline)).squeeze()
         #self._eline_sigma_lambda = eline_sigma_kms * self._ewave_obs / ckms
 
-        # exit gracefully if not fitting lines
-        if (obs.get('spectrum', None) is None):
-            self._elines_to_fit = None
-            self._eline_wavelength_mask = np.array([], dtype=bool)
-            return
+        # --- get valid lines ---
+        # fixed and fit lines specified by user, but remove any lines which do
+        # not have an observed pixel within 5sigma of their center
+        # This part has to go in every call
+        linewidth = nsigma * self._ewave_obs / ckms * self._eline_sigma_kms
+        pixel_mask = (np.abs(self._outwave - self._ewave_obs[:, None]) < linewidth[:, None])
+        self._valid_eline = pixel_mask.any(axis=1) & self._use_eline
 
-        # --- lines to fit ---
-        # lines specified by user, but remove any lines which do not
-        # have an observed pixel within 5sigma of their center
-        eline_names = self.params.get('lines_to_fit', [])
-
-        # FIXME: this should be moved to instantiation and only done once
-        SPS_HOME = os.getenv('SPS_HOME')
-        emline_info = np.genfromtxt(os.path.join(SPS_HOME, 'data', 'emlines_info.dat'),
-                                    dtype=[('wave', 'f8'), ('name', 'S20')],
-                                    delimiter=',')
-        # restrict to specific emission lines?
-        if (len(eline_names) == 0):
-            elines_index = np.ones(emline_info.shape, dtype=bool)
-        else:
-            elines_index = np.array([True if name in eline_names else False
-                                     for name in emline_info['name']], dtype=bool)
-        eline_sigma_lambda = self._ewave_obs / ckms * self._eline_sigma_kms
-        new_mask = np.abs(self._outwave-self._ewave_obs[:, None]) < nsigma*eline_sigma_lambda[:, None]
-        self._elines_to_fit = elines_index & new_mask.any(axis=1)
-
-        # --- wavelengths corresponding to those lines ---
+        # --- wavelengths corresponding to valid lines ---
         # within N sigma of the central wavelength
-        self._eline_wavelength_mask = new_mask[self._elines_to_fit, :].any(axis=0)
+        self._fit_eline_pixelmask = pixel_mask[self._valid_eline & self._fit_eline, :].any(axis=0)
+        self._fix_eline_pixelmask = pixel_mask[self._valid_eline & self._fix_eline, :].any(axis=0)
+        # --- lines to fit ---
+        self._elines_to_fit = self._fit_eline & self._valid_eline
+
+    def parse_elines(self):
+        """Create mask arrays to identify the lines that should be fit and the
+        lines that should be fixed based on the content of `params["lines_to_fix"]`
+        and `params["lines_to_fit"]`
+
+        This can probably be cached just once unless you want to change between
+        separate fits.
+        """
+
+        all_lines = self.emline_info['name']
+
+        if self.params.get('marginalize_elines', False):
+            # if marginalizing, default to fitting all lines
+            # unless some are explicitly fixed
+            lnames_to_fit = self.params.get('elines_to_fit', all_lines)
+            lnames_to_fix = self.params.get('elines_to_fix', np.array([]))
+            self._fit_eline = np.isin(all_lines, lnames_to_fit) & ~np.isin(all_lines, lnames_to_fix)
+        else:
+            self._fit_eline = np.zeros(len(all_lines), dtype=bool)
+
+        self._fix_eline = ~self._fit_eline
+
+        if self.params.get("elines_to_ignore", []):
+            self._use_eline = ~np.isin(self.emline_info["name"],
+                                       self.params["elines_to_ignore"])
 
     def get_el(self, obs, calibrated_spec, sigma_spec=None):
         """Compute the maximum likelihood and, optionally, MAP emission line
@@ -314,8 +422,9 @@ class SpecModel(ProspectorParams):
             ``(n_wave,)``
 
         :param calibrated_spec:
-            The predicted observer-frame spectrum in the same units as the
-            observed spectrum, ndarray of shape ``(n_wave,)``
+            The predicted (so far) observer-frame spectrum in the same units as
+            the observed spectrum, ndarray of shape ``(n_wave,)``  Should
+            include pixed lines but not lines to be fit
 
         :param sigma_spec:
             Spectral covariance matrix, if using a non-trivial noise model.
@@ -330,12 +439,12 @@ class SpecModel(ProspectorParams):
         """
         # ensure we have no emission lines in spectrum
         # and we definitely want them.
-        assert self.params['nebemlineinspec'] == False
-        assert self.params['add_neb_emission'] == True
+        assert bool(self.params['nebemlineinspec']) is False
+        assert bool(self.params['add_neb_emission']) is True
 
-        # generate Gaussians on appropriate wavelength gride
+        # generate Gaussians on appropriate wavelength grid
         idx = self._elines_to_fit
-        emask = self._eline_wavelength_mask
+        emask = self._fit_eline_pixelmask
         nebwave = self._outwave[emask]
         eline_gaussians = self.get_eline_gaussians(lineidx=idx, wave=nebwave)
 
@@ -348,6 +457,7 @@ class SpecModel(ProspectorParams):
         linecal = units_factor * calib_factor
         alpha_breve = self._eline_lum[idx] * linecal
 
+        # FIXME: nebopt: be careful with inverses
         # generate inverse of sigma_spec
         if sigma_spec is None:
             sigma_spec = obs["unc"]**2
@@ -358,6 +468,7 @@ class SpecModel(ProspectorParams):
             sigma_inv = np.diag(1. / sigma_spec)
 
         # calculate ML emission line amplitudes and covariance matrix
+        # FIXME: nebopt: do this with a solve
         sigma_alpha_hat = np.linalg.pinv(np.dot(eline_gaussians.T, np.dot(sigma_inv, eline_gaussians)))
         alpha_hat = np.dot(sigma_alpha_hat, np.dot(eline_gaussians.T, np.dot(sigma_inv, delta)))
 
@@ -370,6 +481,7 @@ class SpecModel(ProspectorParams):
             alpha_bar = (np.dot(sigma_alpha_breve, np.dot(M, alpha_hat)) +
                          np.dot(sigma_alpha_hat, np.dot(M, alpha_breve)))
             sigma_alpha_bar = np.dot(sigma_alpha_hat, np.dot(M, sigma_alpha_breve))
+            # FIXME: nebopt: can we avoid a scipy call here
             K = ln_mvn(alpha_hat, mean=alpha_breve, cov=sigma_alpha_breve+sigma_alpha_hat) - \
                 ln_mvn(alpha_hat, mean=alpha_hat, cov=sigma_alpha_hat)
         else:
@@ -386,20 +498,24 @@ class SpecModel(ProspectorParams):
         # return the maximum-likelihood line spectrum in observed units
         return alpha_hat * eline_gaussians
 
-    def get_eline_spec(self, wave=None):
+    def predict_eline_spec(self, line_indices=slice(None), wave=None):
         """Compute a complete model emission line spectrum. This should only
         be run after calling predict(), as it accesses cached information.
         Relatively slow, useful for display purposes
 
+        :param line_indices: optional
+            If given, this should give the indices of the lines to predict.
+
         :param wave: (optional, default: ``None``)
             The wavelength ndarray on which to compute the emission line spectrum.
             If not supplied, the ``_outwave`` vector is used.
 
         :returns eline_spec:
-            An (n_line, n_wave) ndarray
+            An (n_line, n_wave) ndarray, units of Lsun/Hz intrinsic (no
+            calibration vector applied)
         """
-        gaussians = self.get_eline_gaussians(wave=wave)
-        elums = self._eline_lum * self.flux_norm() / (1 + self._zred)
+        gaussians = self.get_eline_gaussians(lineidx=line_indices, wave=wave)
+        elums = self._eline_lum[line_indices] * self.flux_norm() / (1 + self._zred)
         return elums * gaussians
 
     def get_eline_gaussians(self, lineidx=slice(None), wave=None):
@@ -474,6 +590,42 @@ class SpecModel(ProspectorParams):
     def spec_calibration(self, **kwargs):
         return np.ones_like(self._outwave)
 
+    def absolute_rest_maggies(self, filters):
+        """Return absolute rest-frame maggies (=10**(-0.4*M)) of the last
+        computed spectrum.
+
+        Parameters
+        ----------
+        filters : list of ``sedpy.observate.Filter()`` instances
+            The filters through which you wish to compute the absolute mags
+
+        Returns
+        -------
+        maggies : ndarray of shape (nbands,)
+            The absolute restframe maggies of the model through the supplied
+            filters, including emission lines.  Convert to absolute rest-frame
+            magnitudes as M = -2.5 * log10(maggies)
+        """
+        # --- convert spectrum ---
+        ld = cosmo.luminosity_distance(self._zred).to("pc").value
+        # convert to maggies if the source was at 10 parsec, accounting for the (1+z) applied during predict()
+        fmaggies = self._norm_spec / (1 + self._zred) * (ld / 10)**2
+        # convert to erg/s/cm^2/AA for sedpy and get absolute magnitudes
+        flambda = fmaggies * lightspeed / self._wave**2 * (3631*jansky_cgs)
+        abs_rest_maggies = 10**(-0.4 * np.atleast_1d(getSED(self._wave, flambda, filters)))
+        # TODO: below is faster for sedpy > 0.2.0
+        #abs_rest_maggies = np.atleast_1d(getSED(self._wave, flambda, filters, linear_flux=True))
+
+        # add emission lines
+        if bool(self.params.get('nebemlineinspec', False)) is False:
+            eline_z = self.params.get("eline_delta_zred", 0.0)
+            elams = (1 + eline_z) * self._eline_wave
+            elums = self._eline_lum * self.flux_norm() / (1 + self._zred) * (3631*jansky_cgs) * (ld / 10)**2
+            emaggies = self.nebline_photometry(filters, elams=elams, elums=elums)
+            abs_rest_maggies += emaggies
+
+        return abs_rest_maggies
+
     def mean_model(self, theta, obs, sps=None, sigma=None, **extras):
         """Legacy wrapper around predict()
         """
@@ -488,6 +640,13 @@ class PolySpecModel(SpecModel):
     spectrum.
     """
 
+    def _available_parameters(self):
+        pars = [("polyorder", "order of the polynomial to fit"),
+                ("poly_regularization", "vector of length `polyorder` providing regularization for each polynomial term")
+                ]
+
+        return pars
+
     def spec_calibration(self, theta=None, obs=None, spec=None, **kwargs):
         """Implements a Chebyshev polynomial calibration model. This uses
         least-squares to find the maximum-likelihood Chebyshev polynomial of a
@@ -497,7 +656,7 @@ class PolySpecModel(SpecModel):
         least-squares fit.
 
         :returns cal:
-           A polynomial given by :math:`\Sum_{m=0}^M a_{m} * T_m(x)`.
+           A polynomial given by :math:`\sum_{m=0}^M a_{m} * T_m(x)`.
         """
         if theta is not None:
             self.set_parameters(theta)
@@ -512,7 +671,7 @@ class PolySpecModel(SpecModel):
             # remove region around emission lines if doing analytical marginalization
             mask = obs.get('mask', np.ones_like(obs['wavelength'], dtype=bool)).copy()
             if self.params.get('marginalize_elines', False):
-                mask[self._eline_wavelength_mask] = 0
+                mask[self._fit_eline_pixelmask] = 0
 
             # map unmasked wavelengths to the interval -1, 1
             # masked wavelengths may have x>1, x<-1
@@ -536,6 +695,79 @@ class PolySpecModel(SpecModel):
         return (1.0 + poly)
 
 
+class SplineSpecModel(SpecModel):
+
+    """This is a subclass of *SpecModel* that generates the multiplicative
+    calibration vector at each model `predict` call as the maximum likelihood
+    cubic spline describing the ratio between the observed and the model
+    spectrum.
+    """
+
+    def _available_parameters(self):
+        pars = [("spline_knot_wave", "vector of wavelengths for the location ot he spline knots"),
+                ("spline_knot_spacing", "spacing between knots, in units of wavelength"),
+                ("spline_knot_n", "number of interior knots between minimum and maximum unmasked wavelength")
+                ]
+
+        return pars
+
+    def spec_calibration(self, theta=None, obs=None, spec=None, **kwargs):
+        """Implements a spline calibration model.  This fits a cubic spline with
+        determined knot locations to the ratio of the observed spectrum to the
+        model spectrum.  If emission lines are being marginalized out, they are
+        excluded from the least-squares fit.
+
+        The knot locations must be specified as model parameters, either
+        explicitly or via a number of knots or knot spacing (in angstroms)
+        """
+        if theta is not None:
+            self.set_parameters(theta)
+
+        splineopt = True
+        if splineopt:
+            mask, (wlo, whi) = self.obs_to_mask(obs)
+            y = (obs['spectrum'] / spec)[mask] - 1.0
+            yerr = (obs['unc'] / spec)[mask]  # HACK
+            knots_x = self.make_knots(wlo, whi, as_x=True, **self.params)
+            x = 2.0 * (obs["wavelength"] - wlo) / (whi - wlo) - 1.0
+            tck = splrep(x[mask], y[mask], w=1/yerr[mask], k=3, task=-1, t=knots_x)
+            self._spline_coeffs = tck
+            spl = BSpline(*tck)
+            spline = spl(x)
+        else:
+            spline = np.zeros_like(self._outwave)
+        return (1.0 + spline)
+
+    def make_knots(self, wlo, whi, as_x=True, **params):
+        """
+        """
+        if "spline_knot_wave" in params:
+            knots = np.squeeze(params["spline_knot_wave"])
+        elif "spline_knot_spacing" in params:
+            s = np.squeeze(params["spline_knot_spacing"])
+            # we drop the start so knots are interior
+            knots = np.arange(wlo, whi, s)[1:]
+        elif "spline_knot_n" in params:
+            n = np.squeeze(params["spline_knot_n"])
+            # we need to drop the endpoints so knots are interior
+            knots = np.linspace(wlo, whi, n)[1:-1]
+        else:
+            raise KeyError("No valid spline knot specification in self.params")
+
+        if as_x:
+            knots = 2.0 * (knots - wlo) / (whi - wlo) - 1.0
+
+        return knots
+
+    def obs_to_mask(self, obs):
+        mask = obs.get('mask', np.ones_like(obs['wavelength'], dtype=bool)).copy()
+        if self.params.get('marginalize_elines', False):
+            mask[self._fit_eline_pixelmask] = 0
+        w = obs["wavelength"]
+        wrange = w[mask].min(), w[mask].max()
+        return mask, wrange
+
+
 class SedModel(ProspectorParams):
 
     """A subclass of :py:class:`ProspectorParams` that passes the models
@@ -694,7 +926,7 @@ class PolySedModel(SedModel):
         normalization is controlled by ``spec_norm``.
 
         :returns cal:
-           A polynomial given by 'spec_norm' * (1 + \Sum_{m=1}^M a_{m} * T_m(x)).
+           A polynomial given by 'spec_norm' * (1 + \sum_{m=1}^M a_{m} * T_m(x)).
         """
         if theta is not None:
             self.set_parameters(theta)
