@@ -114,13 +114,19 @@ def add_dynesty_args(parser):
                         help="If set, do nested sampling with dynesty.")
 
     parser.add_argument("--nested_bound", type=str, default="multi",
+                        choices=["single", "multi", "balls", "cubes"],
                         help=("Method for bounding the prior volume when drawing new points. "
                               "One of single | multi | balls | cubes"))
 
-    parser.add_argument("--nested_method", type=str, default="slice",
+    parser.add_argument("--nested_sample", "--nested_method", type=str, dest="nested_sample",
+                        default="slice", choices=["unif", "rwalk", "slice"],
                         help=("Method for drawing new points during sampling.  "
                               "One of unif | rwalk | slice"))
 
+    parser.add_argument("--nested_walks", type=int, default=48,
+                        help=("Number of Metropolis steps to take when "
+                              "`nested_sample` is 'rwalk'"))
+
     parser.add_argument("--nlive_init", dest="nested_nlive_init", type=int, default=100,
                         help="Number of live points for the intial nested sampling run.")
 
